# $Id: Project.pm,v 1.4 2002/01/28 19:49:13 joern Exp $

#-----------------------------------------------------------------------
# Copyright (C) 2001-2002 Jrn Reder <joern@zyn.de> All Rights Reserved
# 
# This program is part of Video::DVDRip, which is free software; you can
# redistribute it and/or modify it under the same terms as Perl itself.
#-----------------------------------------------------------------------

package Video::DVDRip::Cluster::Project;

use base Video::DVDRip::Project;

use Video::DVDRip::Cluster::Title;

use Carp;
use strict;

sub id				{ shift->{id}				}
sub state			{ shift->{state}			}
sub assigned_job		{ shift->{assigned_job} 		}

sub set_id			{ shift->{id}			= $_[1] }
sub set_assigned_job		{ shift->{assigned_job} 	= $_[1] }

sub vob_dir {
	my $self = shift;
	
	my $job = $self->assigned_job or croak "No job assigned";

	return $job->node->data_base_dir."/".
	       $self->name."/vob";
}

sub avi_dir {
	my $self = shift;
	
	my $job  = $self->assigned_job or croak "No job assigned";
	my $node = $job->node;

	return $node->data_base_dir."/".
	       $self->name."/cluster/".
	       $node->name;
}

sub final_avi_dir {
	my $self = shift;
	
	my $job  = $self->assigned_job or croak "No job assigned";
	my $node = $job->node;

	return $node->data_base_dir."/".
	       $self->name."/avi";
}

sub snap_dir {
	my $self = shift;
	
	my $job = $self->assigned_job or croak "No job assigned";

	return $job->node->data_base_dir."/".
	       $self->name."/tmp";
}

my %KNOWN_STATES = (
	'not scheduled'        => { '' 			   => 1	},
	'waiting'   	       => { 'not scheduled' 	   => 1,
				    'transcoding'	   => 1,
				    'merging'		   => 1,},
	'transcoding'          => { 'waiting'   	   => 1	},
	'transcoded'	       => { 'transcoding'	   => 1	},
	'merging'	       => { 'transcoded'	   => 1 },
	'merged'	       => { 'merging'		   => 1 },
	'audio processing'     => { 'merged'	 	   => 1	},
	'finished'	       => { 'audio processing'	   => 1	},
);

sub set_state {
	my $self = shift;
	my ($state) = @_;
	
	croak "Unknown state '$state'"
		if not defined $KNOWN_STATES{$state};

	my $old_state = $self->state;
	
	croak "Illegal state change from '$old_state' to '$state'"
		if $state ne $old_state and
		   not $KNOWN_STATES{$state}->{$old_state};

	$self->{state} = $state;
	
	return $state;
}

sub force_state {
	my $self = shift;
	my ($state) = @_;
	
	croak "Unknown state '$state'"
		if not defined $KNOWN_STATES{$state};
	
	$self->{state} = $state;
	
	return $state;
}

sub title {
	my $self = shift;
	
	return $self->content->titles->{$self->selected_title_nr};
}

sub label {
	my $self = shift;
	return $self->name." (#".$self->selected_title_nr.")";
}

sub new {
	my $class = shift;
	my %par = @_;
	my ($project, $title_nr) = @par{'project','title_nr'};
	
	# bless instance with this class
	bless $project, $class;
	
	# bless all titles with the Cluster::Title class
	if ( $project->content->titles ) {
		foreach my $title ( values %{$project->content->titles} ) {
			bless $title, "Video::DVDRip::Cluster::Title";
		}
	}
	
	return $project;
}

sub reset {
	my $self = shift;
	
	my $state = $self->state;
	
	if ( $state eq 'transcoding' ) {
		$self->force_state ('waiting');

	} elsif ( $state eq 'merging' ) {
		$self->force_state ('transcoded');

	} elsif ( $state eq 'audio processing' ) {
		$self->force_state ('merged');
	}

	$self->title->reset_chunks;

	1;
}

sub get_save_data {
	my $self = shift;
	
	# don't save current job assignement
	my $job = $self->assigned_job;
	$self->set_assigned_job(undef);
	
	# get save data by calling super method
	my $data = $self->SUPER::get_save_data;
	
	# restore current job assignement
	$self->set_assigned_job($job);

	return $data;
}

sub progress {
	my $self = shift;
	
	if ( $self->state eq 'transcoding' ) {
		# transcoding progress
		my $title  = $self->title;
		my $frames = $title->frames_finished;
		my $fps = 0;

		my $nodes = Video::DVDRip::Cluster::Job->get_project_nodes (
			project => $self,
		);

		foreach my $node ( values %{$nodes} ) {
			$frames += $node->progress_frames;
			$fps    += $node->progress_fps;
		}

		return sprintf (
			"%2.2f\%, %2.1f fps, %d/%d chunks",
			$frames / $title->frames * 100,
			$fps,
			$title->chunks_finished,
			$title->chunk_cnt
		);

	} elsif ( $self->state eq 'merging' or
		  $self->state eq 'audio processing' ) {
		# merging or audio processing progress
		my $title  = $self->title;
		my $nodes = Video::DVDRip::Cluster::Job->get_project_nodes (
			project => $self,
		);

		# these jobs run on one node, so we copy its
		# progress state
		my ($node) = values %{$nodes};
		return $node->progress;		

	} else {
		return "";
	}
}

1;
